/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_starthack.c: Allows admins to hack server start time.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  m_starthack.c - (C) Alan 'alz' Milford, 10th May 2003. (alz@froginternet.com)
 *
 *  Usage Details:
 * 
 *  This command is available for use by server operators (+o flag),
 * 
 *  /servstartts <TS>      - Sets server start time to <TS>
 *  /resetstartts          - Resets the start TS to the previous value.
 *
 * 
 *  Note: This is POINTLESS and vanity. Only use it if you're a moron.
 *
 *                                                        -alz
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "whowas.h"
#include "irc_string.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "list.h"

int oldstart;

/* SND_HSTART_WALLOPS
 *
 * defining this will send a wallops message when an admin hacks the
 * server's start time.
 *
 */
#define SND_HSTART_WALLOPS

static void mo_servstartts(struct Client *, struct Client *, int, char**);
static void mo_resetstartts(struct Client *, struct Client *, int, char**);

struct Message servstartts_msgtab = {
  "SERVSTARTTS", 0, 0, 1, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_servstartts}
};

struct Message resetstartts_msgtab = {
  "RESETSTARTTS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_resetstartts}
};

#ifndef STATIC_MODULES

void
_modinit(void)
{
  mod_add_cmd(&servstartts_msgtab);
  mod_add_cmd(&resetstartts_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&servstartts_msgtab);
  mod_del_cmd(&resetstartts_msgtab);
}

const char *_version = "1.5";
#endif


/*
** mo_servstartts
**      parv[0] = sender prefix
**      parv[1] = new server start TS
*/
static void mo_servstartts(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
  int newstart = 0;
  if (!IsOperAdmin(source_p))
    {
      sendto_one(source_p, ":%s NOTICE %s :You are not teh admined.", me.name, parv[0]);
      return;
    }
  if (parv[1] == NULL)
    {
      sendto_one(source_p, ":%s NOTICE %s :No TS specified", me.name, parv[0]);
      return;
    }
  newstart = atoi(parv[1]);
  if (newstart > me.since) 
    {
     sendto_one(source_p, ":%s NOTICE %s :[setstart] %u is after server start (%lu)", me.name, parv[0], newstart, me.since);     
    } 
  else 
    {
     oldstart = me.since;
     me.since = newstart;
     sendto_one(source_p, ":%s NOTICE %s :[setstart] set: %u -> %u", me.name, parv[0], oldstart, newstart);  
     #ifndef SND_HSTART_WALLOPS 
        sendto_wallops_flags(FLAGS_WALLOP, &me,
              "[setstart] [%s!%s@%s] set: %u -> %u",
              source_p->name, source_p->username, source_p->host, oldstart, newstart);
     #endif  
     }
}

/*
** mo_resetstartts
**      parv[0] = sender prefix
*/
static void mo_resetstartts(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
  int newstart;
  if (!IsOperAdmin(source_p))
    {
      sendto_one(source_p, ":%s NOTICE %s :You are not teh admined.", me.name, parv[0]);
      return;
    }
   newstart = oldstart;
   sendto_one(source_p, ":%s NOTICE %s :[setstart] start reset: %lu -> %u", me.name, parv[0], me.since, newstart);  
     #ifndef SND_HSTART_WALLOPS 
        sendto_wallops_flags(FLAGS_WALLOP, &me,
              "[setstart] [%s!%s@%s] start reset: %u -> %u",
              source_p->name, source_p->username, source_p->host, me.since, newstart);
     #endif  
    me.since = newstart;
}